c===========================================================================
c     routine IP: Instrumental profile

      subroutine IP(g,f,f2,fi,sizfi,sizg,sizf,wlg,wlf,spectrumfn,path,
&	coun,outputfn,n_analyzed)

c===========================================================================
c	20080809: final RV = averaging over sigma clip 2.0 -> 3.3!
c
c	20080722: read sigma clip from file "clip"
c	20080521: enlarge IP array sizfi!
c
c	20080513: using lambda values from IRAF and no header anymore
c	200805: HRS X version clone of cs23 version
c	cs23 VERSION clone of U V E S vs. from 03/06/2002 
c	Version 4.whatever
c
c	FULL MODELING!!!!!
c
c	using input-file INPUT_MAIN (CATALOG)
c
c
c===========================================================================
c     This routine first calculates the matricial elements ((mF)) from the 
c     system g= mF.fi, using the vectors g and f.
c     The IP vecmain_ip.ftor fi is determinated by the ME method.
c     The matrix elements ((mFI)) are then calculated, in order to compute
c     the researched intrinsic spectrum f using the ME method.
c 
c------------------------------
c     description of variables:
c
c     g    Real 1-D arrayip  Discrete observed spectrum vector      Input  
c     f    Real 1-D array  Known discrete intrinsic spectrum vector Input
c     f2   Real 1-D array  Wanted intrinsic spectrum vector         Output 
c     fi   Real 1-D array  Kernel func.= instrumental profile       Output
c     matF Real 2-D array  Intrisic spectrum matrix                 Output
c     matFI Real 2-D array IP response matrix                       Output
c     p    Integer         Cut-off on fi                            Input
c     sizg Integer         Size of g vector (px)                    Input
c     sizf Integer         Size of f vector (px)                    Input
c     wlg  Integer         Starting wavelength of vector g          Input
c     wlf  Integer         Starting wavelength of vector f          Input
c     spectrumfn           Name of dataspectrum            	    Input
c     path		   Path to dataspectrum                     Input
c     coun		   Number of spectrum in catalog 	    Input
c     outputfn		   Name of outputfile			    Input
c     n_analyzed           Number of analyzed spectra               Output
c 		
c     st_flag Integer         star flag (0=no/1=yes)
c	
c==== Local variables
c    
c     i    Integer         observed spectrum vector indice (px)  [1..n]
c     j    Integer         Intrinsic spectrum vector indice (px) [1..qn]
c     q    Real (Integer?) Oversampling factor; q=j/i
c     k,l,m,n              integer sommation indices
c    
c
c===========================================================================

        IMPLICIT NONE

	Integer maxsiz,nb_params,matsiz
	Real*8 const_wl_g

	parameter (maxsiz=1000,nb_params=40)
c	parameter (maxsiz=5000,nb_params=30,matsiz=5000)
	
	parameter (const_wl_g=0.022965999320)

        integer i,j,k,l,m,n,p,q,sizg,sizf,sizfi,n1,niter
&      ,choice1,choice2,sizdfi,wldf,sizgf,wldfi,cutg,aff
        real sum,x1,x2,x3,eta,gmax,factor,tstM,sumratio,sumdfi
        real matF(maxsiz,maxsiz),matFI(maxsiz,maxsiz),signoise
c	real matF(matsiz,matsiz),matFI(matsiz,matsiz),signoise

c       real g(maxsiz),g1(maxsiz),f(maxsiz),fi(maxsiz),X(maxsiz)
	real g(maxsiz),g1(maxsiz),fi(maxsiz),X(maxsiz)
	real f(5000)
c	UVES:
	REAL mask(maxsiz)
	INTEGER chunkgut,npix(1000)

        real fiinv(maxsiz),f2(maxsiz),s(maxsiz),gnoise(maxsiz),
&      dfi(maxsiz),dfi0(maxsiz),fimem(maxsiz)
ccc	real par(nb_params)
        real ratio(maxsiz),deff(maxsiz),wlf
	real*8 wlg,wlg2,wn_thor(1000),step1(1000),step2(1000)
	REAL*8 wl_thor(1000),least,leastmin
	REAL par(nb_params),oldpar(nb_params),clip

	real summin,etamin,niterm,etamax,stepet,sumf,shift
        real stddev,phimax,chi2,gdummy,dummy,oldchi2,chi2sum
        real*8 wng

	integer count,sizgsimu,loop,header
	integer chunk,chunknumber,npixel,startpixel
	integer st_flag,outie
	integer endpixel

	real*8 a0,a1,a2,temppix,a0wn,a1wn,a2wn	
	real*8 a1wn_sav,a2wn_sav

c	UVES:
	REAL*8 disp1(199),disp2(199),disp3(199),disp4(199),disp5(199)
	REAL*8 disp6(199),disp7(199),disp8(199),disp9(199)
	REAL*8 a3,a4,a5,a6,a7,a8

	INTEGER order,startorder,endorder
	REAL s2n_order(199)

	REAL*8 radvel(3000),radvelsum,radvelmean,radvelvar,radvelrms
	REAL*8 radvelmeanerr,radresid,radvel2(3000),radmedian
	REAL*8 barycorr
	REAL t_exp,s2n,lambda(1000)
	REAL*8 BJD,GEO_JD

c       new averaging of sigma clips:
        REAL*8 rvsaf(99),rvmsaf(99),errsaf(99),rmssaf(99)
        REAL*8 rvfinal,errfinal,rmsfinal,rvmfinal
c        REAL clip
        INTEGER zeler,chunksaf


c	Weight on the errorspectrum by the gradient: 
	REAL grad1(1000),grad2(1000),gradient(1000),weight(1000)
	REAL gradmax,gradsum,gradient2(1000)

	INTEGER coun,weather,quality,n_analyzed
	INTEGER totalchunks

c       median:
        INTEGER NR,IR
        REAL XR(2000),XMED,N2,RA(2000)
        REAL RRA,IM,JM,LM

	character*30 outputfn

	real pixel,stpixel

        character*25 namfilfi,nmfresfi,namefile,namespectra,nameerror
        character*1 rep,dummi
	CHARACTER*50 spectrumfn
	CHARACTER*30 keyword1,keyword2,keyword3
	CHARACTER*30 outfn
	CHARACTER*6 path


c=========================================================================

	write(*,*)'*********** IP_6 ***********'
	write(*,*)
c	write(*,*)'Spectrum to process:',spectrumfn
c	write(*,*)
c	write(*,*)'Correct path:',path
c	read(*,*)rep

c=========================================================================

	chunk=1
	st_flag=0
c	pixel=5.
c	pixel=210.

	totalchunks=0

ccccc	Correct starting pixel in spectrum:
c	new version using startpixel value

	pixel=1.

c       HRS start Echelle order:
c	order=21

c       UCLES start spectral order:
        order=1


c===================================================================
c	INPUT FILE
c===================================================================


	write(*,*)'-> Reading the INPUT file.......'

c	open(unit=13, file='./data/input2', type='old')

	open(13,file='./data/input_main', type='old')

c	do i=1,7
	do i=1,15
		read(13,*) rep
	end do


c-----  Default IP:

	read(13,*) rep
	read(13,*) namefile
c	write(*,*)'OK, default IP solution=',namefile

c	namefile='dfi49.asc'
c	namefile='firstip.asc'
c	namefile='testphi.asc'
c	namefile='gsimdef.asc'
c	namefile='gsimip2.asc'


        open (25,file='./data/'//namefile,type='old')
        read (25,*) sizdfi,wldf
c        write(*,*) ' Reading the default IP...'
c        write (*,*) ' Sizdfi :',sizdfi

        sumdfi=0.
        do k=1,sizdfi
           read (25,*) dfi(k)
           dfi0(k)=dfi(k)
            sumdfi=sumdfi+dfi(k)
        end do
c        write(*,*) ' Sum dfi:',sumdfi
        close(25)

	do k=1,sizdfi
		dfi(k)=dfi(k)/sumdfi
	end do

	choice1=2

        if (choice1.ne.1) then

c-- Acquisition of the observed data


ccc	read(13,*) rep
ccc	read(13,*) namespectra
ccc	write(*,*)'OK, name of observed IODINE spectrum = ',namespectra

c---	Header lines:

ccc	read(13,*) rep
ccc	read(13,*) header
ccc	write(*,*)'OK, number of header lines=',header



	read(13,*)rep
	read(13,*)npixel

c	write(*,*)'Total number of pixel per order:',npixel

	read(13,*)rep
	read(13,*)startpixel,endpixel

c	write(*,*)'Starting within UVES order with pixel:',startpixel

	read(13,*) rep
	read(13,*) startorder
	write(*,*) 'Starting with spectral order:', startorder

	read(13,*) rep
	read(13,*) endorder
	write(*,*) 'Ending with spectral order:', endorder

c	closing "input_main"
	close(13)

c	Preparing outputfiles:

	outfn=spectrumfn

c	comment this line to run faster???
        open(36, file='./plot/chunk/wave.dat')
        open(15, file='./plot/chunk/'//outfn//'_chunks.dat')
        open(37, file='./plot/chunk/'//outfn//'_rv1.dat')   
	
c	OPEN(71, file='./plot/chunk/newdisp1.dat')

cc	OPEN(unit=17,file='./plot/chunk/'//outfn//'_rv1.dat')	

c	WRITE(*,*)'Outputfn=',outfn//'_rv1.dat'

c--------------------------------------------------------------------------------------------
c--------------------------------------------------------------------------------------------
c-----------------------	OPENING DATA FILE:  -----------------------------------------
c--------------------------------------------------------------------------------------------
c--------------------------------------------------------------------------------------------

	outfn=spectrumfn

	OPEN(22,file='/data2/austral/mike/UCLES/data/'
& //path//'/'//spectrumfn)

c        OPEN(22,file='/data2/austral/mike/HRS/data/'
c& //path//'/'//spectrumfn)

c------------

c        write(*,*) 'Reading the observed spectrum file='
c	write(*,*)spectrumfn

c----	Reading Header information
	
	read(22,*)dummi
	read(22,*)dummi
	read(22,*)dummi
	read(22,*)dummi

c---	Exposure Time:
	read(22,*)dummi

c
c	read(22,70)keyword1,t_exp
c	
c	write(*,*)'Exposure time= ',t_exp
c
70	format(A21,E10.2)

c---	Barycorr:

	read(22,*)dummi
	read(22,*)dummi

c	formatted:
c	read(22,72)keyword1,BJD,keyword2,barycorr

c	not formated:
	read(22,*) dummi,GEO_JD,BJD,barycorr

c	OPEN(69,file='datum')
c	WRITE(69,*) BJD
c	CLOSE(69)

c	write(*,*)'B_JD='
c	write(*,109)BJD
c	write(*,*)'BARYCORR='
c	write(*,109)barycorr

72	format(A26,D25.2,A1,D25.2)

	read(22,*)dummi 

c---	Dispersion Solution:

c======= reading all UCLES orders: ===========
c======= new chip:
c======= orders 68 -> 118

	DO j=1,51

c	do j=1,endorder
c	j=1

cc	read(22,74)keyword1,a0,keyword2,a1,keyword3,a2
cc	read(22,74)keyword1,disp1(j),keyword2,disp2(j),keyword3,disp3(j)

c----	2nd order polynomial:
c
c 	try unformatted:
c	read(22,*) dummi
c	read(22,*) dummi,disp1(j),disp2(j),disp3(j)
c
c	Jumping:
c	read(22,*) dummi 
c	read(22,*) dummi 
c	read(22,*) dummi
c       read(22,*) dummi   
c
c----	8th order polynomial:

	read(22,*) dummi
	read(22,*) dummi
	read(22,*) dummi

	read(22,*) dummi,disp1(j),disp2(j),disp3(j)
	read(22,*) dummi,disp4(j),disp5(j),disp6(j) 
	read(22,*) dummi,disp7(j),disp8(j),disp9(j) 

c74	format(A2,D24.2,A1,D24.2,A1,D25.2) 

c---

74     format(A2,D24.2,A1,D24.2,A1,D25.2)

c	write(*,*)'DISPERSION ORDER#',j,': A0,A1,A2='
c	write(*,109) disp1(j)
c	write(*,109) disp2(j)
c	write(*,109) disp3(j)
c	write(*,109) disp4(j)
c       write(*,109) disp5(j)
c        write(*,109) disp6(j)  
c 	write(*,109) disp7(j)
c        write(*,109) disp8(j)
c        write(*,109) disp9(j)  

c---	S/N

	read(22,*)dummi

	read(22,*) dummi,s2n_order(j)

c	read(22,76)keyword1,s2n_order(j)
c	write(*,*)'mean S/N Order#',j,':',s2n_order(j)

76	format(A2,E11.2)

	end do
c========= end of j-loop (all orders) ======================


c---
c	read(22,*)dummi
c       read(22,*)dummi

	read(22,78)keyword1,weather
	read(22,78)keyword1,quality

78	format(A13,I1)

	write(*,*)'Weather flag =',weather
	write(*,*)'Quality flag =',quality

c---
c	Check whether quality is OK
c	stop if quality=2 (spectrum=crap)

	if (quality.eq.2) then
		write(*,*)'QUALITY FLAG = 2 !!'
		write(*,*)'STOPPING!'
c		next spectrum:
		close(22)
c		close(13)
		open(13,file='./plot/chunk/'//spectrumfn//'.warning')
		write(13,*)'QUALITY FLAG = 2 !! SPECTRUM NOT PROCESSED'
c		close(13)
		GOTO 9000
	end if

c---

        read(22,*)dummi
        read(22,*)dummi
	read(22,*)dummi
        read(22,*)dummi   
	read(22,*)dummi
        read(22,*)dummi   
	read(22,*)dummi
        read(22,*)dummi   
	read(22,*)dummi

c----------------------------------------------------------------------------
c
c	JUMPING OVER ORDERS IF STARTORDER<>1
c
c----------------------------------------------------------------------------

	WRITE(*,*)'Start order= ',startorder

	IF (startorder.ne.1) THEN

	WRITE(*,*)' OK, JUMPING OVER ORDERS UNTIL ORDER#',startorder

	DO j=1,startorder-1

		DO i=1,npixel
			read(22,*)dummi
		END DO
		WRITE(*,*)' Jumping 1x...'

	END DO

	order=startorder

c	just a check:
c	READ(22,*,END=8000) npix(1),lambda(1),g(1),s(1),mask(1) 	
c	WRITE(*,*) 'Here were are now:'
c	WRITE(*,*) npix(1),lambda(1),g(1),s(1),mask(1)


	END IF

c----------------------------------------------------------------------------

c-----	Jumping over first 5 Pixel
c	safetymeasure: sometimes first pixels are crap
c	=> corrupting first chunk

c	write(*,*)'Jumping from 1 -> ',startpixel-1

	do i=1,startpixel-1
		read(22,*)dummy
		pixel=pixel+1.
	end do

	write(*,*)'Size of g-chunk=',sizg


c======================  READING OBSERVED SPECTRUM: ==============================


333	CONTINUE

	write(*,*)'Location in spectrum now: Order#:',order,' Pixel#:',pixel

c	checking end of order?

c	IF (pixel+sizg.gt.npixel) THEN
	IF (pixel+sizg.gt.endpixel) THEN


		WRITE(*,*) '********************************************'
		WRITE(*,*)
		WRITE(*,*) 'OK, cant read complete chunk => NEXT ORDER!'
		WRITE(*,*)
		WRITE(*,*) '********************************************'
		GOTO 444
	END IF

c	jumping over order change (normal case):
	GOTO 555

c=====================================================================================
c
c		N E X T      O R D E R 
c
c=====================================================================================

444	CONTINUE

		order=order+1
		chunk=1

		IF (order.gt.endorder) THEN
			WRITE(*,*) 'OK, END OF SPECTRUM REACHED!'
			GOTO 8000
		END IF

		WRITE(*,*)
		WRITE(*,*) 'CHANGING SPECTRAL ORDER..........'
		WRITE(*,*)

c		WRITE(*,*) 'BARYCORR before:',barycorr

		WRITE(*,*) 'New spectral order= ',order
		WRITE(*,*) 'Total of pixel per order: ',npixel

c		WRITE(*,*) 'That many pixel left in old order:',pixel+sizg-npixel
	
c		WRITE(*,*) 'npixel - pixel = ',npixel-pixel+1
		
		DO i=1,npixel-pixel+1

c	Schaas:!	DO i=1,pixel+sizg-npixel+1

c			read(22,*) dummi

c		READ(22,*,END=8000) npix(i),lambda(i),g(i),s(i),mask(i)
		READ(22,*) npix(i)

c		WRITE(*,*)' HERE WE ARE:'
c		WRITE(*,*) npix(i),lambda(i),g(i),s(i),mask(i)

		END DO		

c		DO i=1,32
c		WRITE(*,*) 'par(',i,')= ',par(i)
c		END DO


		pixel=1

		a0=disp1(order)
		a1=disp2(order)
		a2=disp3(order)
		a3=disp4(order)
                a4=disp5(order)
                a5=disp6(order)     
		a6=disp7(order)
                a7=disp8(order)
                a8=disp9(order)     

c		WRITE(*,*)' New dispersion solution= '
c		WRITE(*,109) a0
c		WRITE(*,109) a1
c		WRITE(*,109) a2

		do i=1,startpixel-1
                	read(22,*)dummy
                	pixel=pixel+1.
	        end do



		GOTO 333

c============ END OF ORDER JUMP ! ===============================

555	CONTINUE

	gmax=0.

c	write(*,*)' ============> Reading chunk....'

	chunkgut=0
	
        do k=1,sizg

c----------- READING SPECTRUM: --------------

c	original:
c	           read(22,*,END=8000) n,lambda,g(k),s(k)
c
c	UVES:
		
	READ(22,*,END=8000) npix(k),lambda(k),g(k),s(k),mask(k)


c	WRITE(*,*) npix(k),lambda(k),g(k),s(k),mask(k)
c--------------------------------------------

c	checking mask: if mask never<>6 then read next chunk!

		if (mask(k).ne.6.) chunkgut=1	

		IF (mask(k).eq.6.) THEN 
			s(k)=g(k)*10.
		END IF

		IF (g(k).eq.0) THEN
				g(k)=0.01
				s(k)=1000.
		END IF

		IF (s(k).eq.0) THEN
			s(k)=1000.
		END IF


c	cosmics?
c		IF (mask(k).eq.1.) s(k)=g(k)*2.

c		WRITE(*,*) npix(k),lambda(k),g(k),s(k),mask(k)

		if (g(k).gt.gmax) then
			gmax=g(k)
		end if

c---	Check error vector !!

c===	for VLC data, where error is frequently 0. :

		if (s(k).eq.0.) then

			s(k)=1000.

		end if
		
cc		if (s(k).eq.0.) then
cc			close(22)
cc			close(13)
cc
cc			open(13,file='./plot/chunk/'//spectrumfn//'.warning')
cc	                write(13,*)'ERROR VECTOR = 0 !! SPECTRUM NOT PROCESSED'
cc      	        close(13)
cc
cc			GOTO 9000
cc		end if

c---

c========================================================================

        END DO

c==================== END OF READING SPECTRUM ===========================

c	WRITE(*,*)' 1st pixel read: '
c	WRITE(*,*) npix(1),lambda(1),g(1),s(1),mask(1)
c	WRITE(*,*)' Last pixel read: '
c	WRITE(*,*) npix(sizg),lambda(sizg),g(sizg),s(sizg),mask(sizg) 

c	IF (lambda(1).le.5008.5) chunkgut=0

	IF (chunkgut.eq.0) THEN

		WRITE(*,*)' CHUNK IS EITHER FULL OF CORRUPTED 
& PIXELS OR BLUER THAN 5000 A !!!'
c		WRITE(*,*)' i.e. mask is always 6 = corrupted or no I2 lines '
		WRITE(*,*)' => reading next chunk...'

		pixel=pixel+sizg

		GOTO 333
 
	END IF

c	WRITE(*,*)'Order = ',order 
c	WRITE(*,*)'HERE!: ',disp1(order),disp2(order),disp3(order)

	a0=disp1(order)
	a1=disp2(order)
	a2=disp3(order)

c	WRITE(*,*)'a0,a1,a2 = ',a0,a1,a2

c	8th order:
	a3=disp4(order)
	a4=disp5(order)
	a5=disp6(order)
	a6=disp7(order)
	a7=disp8(order)
	a8=disp9(order)


cc--------------------------------------------
c	normalizing g-vector:

	do k=1,sizg
		gnoise(k)=g(k)
		g(k)=g(k)/gmax
	end do
c--------------------------------------------

ccc	write(*,*)' g(13) now:',g(13)
ccc	write(*,*)'Factor old/new:',g(13)/gnoise(13)

c---	Renormalizing error vector to fit the normalized data


c******** I COMMENT THIS OUT FOR BILL'S IODINE SPECTRUM 

	do k=1,sizg

		s(k)=s(k)*(g(k)/gnoise(k))

c---	new error enlargement :
c		s(k)=s(k)*1.1
c---

	end do

c******** DON'T FORGET TO PUT IT BACK IN!!!!

c********** SCALE ONE FLAT TO COMBINED *********
c********** FARPS TESTS:
c
c	do k=1,sizg
c		s(k)=s(k)*5.
c	end do
c
c********************************************************

c---

	
c	--- comment next line if you want to put noise in simu data ---
	goto 45

c------------------------------------------------------------------
	write(*,*)'Putting noise in the simulated data....'
	write(*,*)'...creating gnoise'

	call gaussn2(g,gnoise,s,sizg,1)

c	do i=1,sizg
c		write(*,*)gnoise(i)
c	end do

	open(unit=15, file='./data/gsimu300.dat')
	do i=1,sizg
		write(15,*)gnoise(i)
	end do
	close(15)

	write(*,*)'..... gnoise written to gsimu300.dat'
	read(*,*)rep

45	continue

c----------------------------------------------------------------------
        if (choice1.eq.3) then
 
           open(400,file='../data/resfi')
           read(400,*) sizfi,wldfi
           do k=1,sizfi
              read(400,*) fi(k)
           end do
           close(400)
           goto 200
         end if

c        stddev=(stddev)**2/(signoise)**2
c        write(*,*) ' Chi2 = S(g1-g2)**2 / sigma**2 = ',stddev

c Cut-off value for g:2
c         write(*,*)
c	 write(*,*) '* Data cut-off value ?'
c	 read(5,*) cutg

c        write(*,*) ' Troncated datas : Edge-cut for g vector =',cutg
c&      ,' pixels'

c	loop=0
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


c       closing Input file
c        close(13)

c	open(36, file='./plot/chunk/wave.dat')
c	open(15, file='./plot/chunk/chunks.dat')
c	open(37, file='./plot/chunk/'//outfn//'_rv1.dat')

c-------*******  START OF OPTIMIZING  ******---------------

1000	continue

c	Jumping over gradient & weight:
	goto 1110

c------- Gradientspectrum & weight of errors for RVs ---------------

	grad1(1)=0.
	grad2(1)=0.
	grad1(sizg)=0.
	grad2(sizg)=0.
	gradient(1)=1.
	gradient(sizg)=1.


	do i=2,sizg-1
		grad1(i)=ABS(g(i-1)-g(i))
		grad2(i)=ABS(g(i+1)-g(i))
		gradient(i)=((grad1(i)+grad2(i))/2.)+1.
	end do

c	write(*,*)'ABS(g(3)-g(2))=',ABS(g(3)-g(2))
c
c	open(23,file='./plot/chunk/gradtest.dat')
c
c	do i=1,sizg
c		s(i)=s(i)*1.5/(gradient(i)**3.)
cc		s(i)=s(i)*1.5
c	end do

c		weight(i)=s(i)/gradient(i)
c		write(23,*)g(i),s(i),gradient(i),weight(i),s(i)-weight(i)
c	end do
c	close(23)

	write(*,*)'Okay, check gradtest.dat'

	do i=1,sizg
c		s(i)=s(i)/gradient(i)
c	try:
		s(i)=s(i)*gradient(i)
	end do

c	read(*,*)rep

c-------------------------------------------------------------------

c	write(15,*)'#  N      CHI^2     SHIFT      GAUSS-Par.    BOX-Par.'

1110	continue

c	2nd order:
c	wlg=a0+(a1*pixel)+((pixel**2)*a2)


c	write(*,*) 'a0,a1,a2,a3,a4,a5,a6,a7,a8',a0,a1,a2,a3,a4,a5,a6,a7,a8

c	write(*,*) 'pixel= ',pixel

c	8th order:
c	wlg=a0+(a1*pixel)+((pixel**2.)*a2)+
c& ((pixel**3.)*a3)+((pixel**4.)*a4)+((pixel**5.)*a5)+
c& ((pixel**6.)*a6)+((pixel**7.)*a7)+((pixel**8)*a8)

c	new disp:
	wlg = lambda(1)

c	write(*,*)'Wavelength at Chunk-Start=',wlg

c	Check if start-lambda is bluer than FTS start

	IF (wlg.le.4768.474) THEN

		WRITE(*,*)'OK, chunk is too blue! No FTS yet!'
		WRITE(*,*)' => reading next chunk.....'
		pixel=pixel+sizg
		GOTO 333
	END IF


	IF (wlg.ge.6420.0) THEN
                WRITE(*,*)'OK, chunk is too red! No FTS anymore!'
                WRITE(*,*)' => reading next chunk.....'
                pixel=pixel+sizg
                GOTO 333
        END IF


c	write(*,*)'Pixel at end(start) of Chunk:',pixel+FLOAT(sizg-1)        
c	2nd order:   
c	wlg2=a0+(a1*(pixel+FLOAT(sizg-1)))+ 
c& (((pixel+FLOAT(sizg-1))**2.)*a2)

c	8th order:
c        wlg2=a0+(a1*(pixel+FLOAT(sizg-1)))+
c& (((pixel+FLOAT(sizg-1))**2.)*a2)+(((pixel+FLOAT(sizg-1))**3.)*a3)+
c& (((pixel+FLOAT(sizg-1))**4.)*a4)+(((pixel+FLOAT(sizg-1))**5.)*a5)+
c& (((pixel+FLOAT(sizg-1))**6.)*a6)+(((pixel+FLOAT(sizg-1))**7.)*a7)+
c& (((pixel+FLOAT(sizg-1))**8.)*a8)

c	new disp:
	wlg2 = lambda(sizg)	

        write(*,*)'Wavelength at Chunk-End (start in waven)=',wlg2
c	write(*,*)'Wavenumber at Chunk-End (start in reality)=',1./(wlg2*1E-08)


	IF (wlg2.ge.6429.0) THEN
                WRITE(*,*)'OK, chunk is too red! No FTS anymore!'
                WRITE(*,*)' => reading next chunk.....'
                pixel=pixel+sizg
                GOTO 333
        END IF


c-----
c	write(*,*)'Now estimating dispersion coefficents from ThAr...'

c	OPEN(27,file='./plot/chunk/disp.dat')

c	temppix=pixel-1.
	temppix=DBLE(pixel-1.)	

c	write(*,*)'Starting wn calc at pixel:',temppix+1.

c	a0wn=1./(a0*1E-08)
c	a0wn=1./(wlg2*1E-08)	
c--

c	do i=1,sizg

c	temppix=temppix+1.D0           

c	2nd order:
c	wn_thor(sizg+1-i)=1./((a0+a1*temppix+
c&	 (a2*(temppix**2.)))*1D-08)

c	8th order:

c	wn_thor(sizg+1-i)=1./((a0+(a1*temppix)+
c&       (a2*(temppix**2.))+(a3*(temppix**3.))+
c&	(a4*(temppix**4.))+(a5*(temppix**5.))+   
c&	(a6*(temppix**6.))+(a7*(temppix**7.))+
c&	(a8*(temppix**8.)))*1D-08)

c	wn_thor(i)=1./((a0+a1*temppix+(a2*(temppix**2.)))*1E-08)
c	wl_thor(i)= a0+a1*temppix+(a2*(temppix**2.))
c	wl_thor(sizg+1-i)=a0wn+a1wn*temppix+(a2wn*(temppix**2.))	
c	write(*,109) wn_thor(1),wn_thor(2)

c	end do

c	new disp:
	do i=1,sizg
        wn_thor(sizg+1-i)=1.D0/(lambda(i)*1.0D-08)
        end do

c	write(*,*)'pixel is now:',temppix

c	temppix=temppix+1.D0

c	2nd order:
c	a0wn=1./((a0+a1*temppix+(a2*(temppix**2.)))*1D-08)

c	8th order:

c	a0wn=1./((a0+(a1*temppix)+ 
c&	(a2*(temppix**2.))+(a3*(temppix**3.))+ 
c&	(a4*(temppix**4.))+(a5*(temppix**5.))+ 
c&	(a6*(temppix**6.))+(a7*(temppix**7.))+
c&       (a8*(temppix**8.)))*1D-08) 

c	new disp:
	a0wn = wn_thor(1)	

c	write(*,*)'a0 is now:',a0wn

c--

	a1wn=wn_thor(sizg)-wn_thor(sizg-1)

c       new disp:
        a0wn = a0wn - a1wn

	a2wn=0.
	least=0.
	leastmin=9999999.

c	OPEN(71,file='./plot/chunk/newdisp2.dat')

	step2(1)=(a1wn/1000.D0)
	step2(2)=0.0000001
c	step2(2)=0.0000005

c	a1wn=a1wn-15.*step2(1)	
	a1wn=a1wn-50.*step2(1)
	
c	do j=1,30
	do j=1,100
		
		a1wn=a1wn+step2(1)

c		a2wn=a2wn-25.*step2(2)
		a2wn=a2wn-100.*step2(2)

c		do k=1,50
		do k=1,200

		a2wn=a2wn+step2(2)

c		temppix=pixel-1.
		temppix=0.
		least=0.

		do i=1,sizg
		temppix=temppix+1.
		wl_thor(i)=a0wn+a1wn*temppix+(a2wn*(temppix**2.))
		step1(i)=ABS(wn_thor(i)-wl_thor(i))
		least=least+step1(i)**2.
		end do

c		WRITE(71,*) a1wn,a2wn,least
c		write(*,*)least

		if (least.lt.leastmin) then
			leastmin=least
			a1wn_sav=a1wn
			a2wn_sav=a2wn
		end if			

		end do

c		write(*,*)least

		a2wn=0.

	end do

c	CLOSE(71)
	write(*,*)'Least squares=',leastmin

c	WRITE(71,*) order,chunk,a1wn_sav,a2wn_sav,leastmin

	a1wn=a1wn_sav
	a2wn=a2wn_sav
c	write(*,*)'a1wn=',a1wn
c	write(*,*)'a2wn=',a2wn

	par(2)=a1wn
	par(8)=a2wn

	write(*,*)'par(2)=',par(2)
        write(*,*)'par(8)=',par(8)

c	temppix=pixel-1.
	temppix=0.

c	do i=1,sizg
c		temppix=temppix+1.
c		wl_thor(i)=a0wn+a1wn*temppix+(a2wn*(temppix**2.))
c		step1(i)=(wn_thor(i)-wl_thor(i))
c	end do

c		temppix=temppix+1.
c		step1(i)=wn_thor(i-1)-wn_thor(i)
c                step2(i)=step1(i-1)-step1(i)
c		step1(i)=ABS(wl_thor(i-1)-wl_thor(i))
c		step2(i)=(step1(i)-a1)/(temppix**2.)
c	end do

c	temppix=10.
c	wl_thor(1)= a0+a1*temppix+(a2*(temppix**2.))
c	temppix=11.
c	wl_thor(2)= a0+a1*temppix+(a2*(temppix**2.))
c	step1(1)=((wl_thor(2)-wl_thor(1))-a1)/2.
c	temppix=10.
c	step2(1)= (a2*temppix)+(a2/2.)


c	do i=1,sizg
c		write(27,109) wn_thor(i),wl_thor(i),step1(i)
c		write(*,109) step1(1), step2(1)
c		write(*,109) step1(i)	
c	end do

c	CLOSE(27)

c-----

 109	format(3(3x,e25.15))
c	write(*,*)'Wavelength & wavenumbers written to disp.dat'
c	read(*,*)rep

c	WRITE(*,*)'Before PARAMSOPT: parameters are:'
c	WRITE(*,*) par

c=====================================================================
c     Call of the parameter optimization routine
c ====================================================================
        call 
& params_opt(g,sizg,wlg,wlg2,s,dfi,sizdfi,f,par,
& sizf,chunk,chi2,barycorr,order,npix)

c ====================================================================

c	write(*,*)'-----------------------------------------------------'
c	write(*,*)'Returning from Parameter Opt.'

c	write(*,*)'OK baryvel_star=',barycorr

c------ check if I2 FTS is out of range:

	IF (sizf.eq.-1) THEN

		write(*,*)
		write(*,*)'I2 FTS is out of range!' 
		write(*,*)
		GOTO 8000

	END IF

c------ check if stellar mstb is out of range:

	if (sizf.eq.0) then
		write(*,*) 
		write(*,*)'Stellar Mastertable is out of range!'
		write(*,*)

c	UVES: next order or stop?

		IF (order.eq.endorder) THEN

			WRITE(*,*)
			WRITE(*,*)'LAST ORDER REACHED...SHUTDOWN!'
			goto 8000
		END IF

		IF (order.lt.endorder) THEN
		
			WRITE(*,*) 
			WRITE(*,*)'OK, SWITCHING TO NEXT ORDER....'

			pixel=pixel+sizg
			GOTO 444

		END IF

	end if

	if (sizf.eq.1) then
		write(*,*)
		write(*,*)'Stellar Mastertable has not begun yet!'
		write(*,*)'Trying next CHUNK.....'
		goto 6060
	end if		
c------ 



c	write(*,*)'Chunk Nr.',chunk,' processed.....'
c	write(*,*)'Final CHI^2=',chi2
c	write(*,*)'Parameters are=',par
c	write(*,*)'Start Pixel was:',pixel

c	write(*,*)'Stellar flag=',st_flag

c	write(15,*)chunk,chi2,par(1),par(16),par(7),par(13),par(2),par(8) 

c	write(15,*)order,chunk,chi2,par(1),par(6),par(2),par(19),wn_thor(sizg)

	write(15,*)order,chunk,chi2,par(1),par(6),par(2),par(8),pixel

c	write(15,*)order,chunk,chi2,par(1),par(16),par(2),par(6),pixel

	totalchunks=totalchunks+1

c	end if

c	write(15,*)chunk,chi2,ABS(IFIX(par(7))),ABS(par(13))
c	write(15,*)chunk,chi2,ABS(par(14))	
c	write(15,*)chunk,chi2,par(1),par(2),par(4),ABS(IFIX(par(7))),ABS(par(13))
c------	sum of chi2..

	chi2sum=chi2sum+chi2

c------

	if (chunk.gt.1) then
		if (chi2.gt.oldchi2*10.) then

		write(*,*)'!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
		write(*,*)
		write(*,*)'OH OH, I guess this chunk is crap...'
		write(*,*)'replacing parameters with the set from the previous chunk!'
		write(*,*)
		write(*,*)'!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
		write(*,*)
		write(*,*)'Old parameter set:'
		write(*,*) par
	
		do i=1,nb_params

			par(i)=oldpar(i)

		end do

		write(*,*) 
		write(*,*)'New parameter set:' 
		write(*,*) par

		end if


	end if
c---
	do i=1,nb_params
		oldpar(i)=par(i)
	end do

	oldchi2=chi2
c------
	chunk=chunk+1

 6060   CONTINUE
	pixel=pixel+sizg

	write(*,*) 'Order#',order,'Chunk Nr.=',chunk
	write(*,*) 'Start pixel (old+sizg) =',pixel

c---	EOF?

c	if (pixel+sizg.gt.npixel) then
c		write(*,*)'OK, End of input spectrum reached'
c		goto 8000
c	end if

c---

c----------------------- NEXT CHUNK -----------------------

c	write(*,*)'Reading next chunk....'

c	new: jumping to beginning
	GOTO 333

	gmax=0.

	do k=1,sizg
	
c		read(22,*,END=8000)n,g(k)

c	original:
c		read(22,*,END=8000)n,lambda,g(k),s(k)
c	for 1st tests:

		READ(22,*,END=8000) n,lambda,g(k),s(k),mask(k)
		read(22,*,END=8000) n,g(k),s(k)



		if (g(k).gt.gmax) then 
			gmax=g(k)
		end if

c	error zero?

		if (s(k).eq.0) then

			s(k)=1000.
		end if


	end do

c----------------------- NEXT CHUNK -----------------------


c	normalizing:

	do k=1,sizg
		gnoise(k)=g(k)
		g(k)=g(k)/gmax
	end do

c----------- Renormalising errorvector:

	do k=1,sizg
ccc		read(20,*)n,s(k)

c****   JUST NOT FOR BILL'S SPECTRUM: ********
		s(k)=s(k)*(g(k)/gnoise(k))
c*********************************************

c---	new error enlargement:
c		s(k)=s(k)*1.1
c---

	end do

c------ jumping over gradient stuff:
	goto 6416

c-----------------------------------------------------------
c	Gradient Spectrum & weight of errors

        grad1(1)=0.
        grad2(1)=0.
        grad1(sizg)=0.
        grad2(sizg)=0.
        gradient(1)=1.
        gradient(sizg)=1.


        do i=2,sizg-1
                grad1(i)=ABS(g(i-1)-g(i))
                grad2(i)=ABS(g(i+1)-g(i))
                gradient(i)=((grad1(i)+grad2(i))/2.)+1.
        end do

        do i=1,sizg
                s(i)=s(i)*1.5/(gradient(i)**3.)
        end do

c-----------------------------------------------------------
6416	continue	
	goto 1000


c********************************************************************
c
c
c
c
c********************************************************************


c	write(*,*)'Size of f-vector:',sizf
c	write(*,*)'Doing a test convolution....'

	goto 100

c	do i=1,sizf
c		f(i)=0.
c	end do
c	f(100)=1.

77	continue
	call mconv(f,sizf,sizg,q,gnoise,dfi,sizfi,sizgf)



	write(*,*)'Size of conv. vector=',sizgf
	write(*,*)'Value of conv.vector f_conv(13)=',gnoise(13)

c	goto 78

	sizgsimu=0.
        count=1
        j=1

       do i=1,sizgf
                gdummy=gdummy+gnoise(i)
                count=count+1
                if (count.gt.q) then
                        count=1
                        f2(j)=gdummy/q
                        j=j+1
                        sizgsimu=sizgsimu+1
                        gdummy=0.
                end if
        end do
	write(*,*)'Size of conv.vector now=',sizgsimu
	chi2=0.
	do i=1,sizg
		chi2=chi2+((g(i)-f2(i))/s(i))**2.
	end do

	write(*,*)'This Chi2 is:',chi2

	read(*,*)rep 

c78	continue
c	open(900, file='./plot/testf.dat', type='new')
c	do i=1,sizf
c		write(900,*)gnoise(i)
c	end do
c	close(900)

	goto 100

c End if (choice<>1) condition

	else
c <=> (choice=1):

c Check test of the matrix with simulated values of vector fi

      write(*,*) ' *** Name of the reference "fi" vector file ?'
      read(5,*) namfilfi
      namfilfi='../data/'//namfilfi
c       
        open (200,file=namfilfi)
 
        read (200,*) sizfi,wlf

        write (6,*) ' Sizfi,wlf :',sizfi,wlf
        if (sizfi.ne.sizdfi) stop
        sumdfi=0.
        do k=1,sizfi
           read (200,*) fi(k)
	end do
 
c "Inversion" of fi for the matrical operations. fiinv(i)=fi(sizfi+1-i)
        do i=1,sizfi
           fiinv(i)=fi(sizfi+1-i)
c           write (6,*) i,fi(i),fiinv(i)
        enddo       

        call normalisation(fiinv,sizfi,1.)
        call normalisation(fi,sizfi,1.)

c*** Rewrite normalized fi:
        
        write (200,*) sizfi,wlf
        if (sizfi.ne.sizdfi) stop
        sumdfi=0.
        do k=1,sizfi
           write (200,*) fi(k)
	end do

        call matvecmul(g,matF,fiinv,sizg,sizfi)
c        write(*,*) sizg,sizfi
 
        open(210,file='../data/g.asc')
        write(210,*) sizg,wlf
        
        gmax=0.
        do i=1,sizg

           sum=0.
           do k=q*(i-1)+1,q*i       
               do j=k-p,k+p
                if (j.gt.0.and.j.le.sizf) then
c               write(*,*) 'j ,.., fi () :',j,k-j+p+1,fi(k-j+p+1)
                 sum=sum+f(j)*fi(k-j+p+1)
                 end if

               end do
            end do
            g1(i)=sum
            if (g1(i).lt.0.99*g(i).or.g1(i).gt.1.01*g(i)) then
               write(*,*) ' !!!!! g1<>g :'
            write(*,*) ' i: ',i,'  g1 =',g1(i),' g =',g(i)
            end if
            write(210,*) g(i)          
            if (g(i).gt.gmax) then
               gmax=g(i)
            end if
         end do

ccc      write(*,*) ' p, 2*p+1 =',p,2*p+1,'** ok4'

          close(210)

ccc Computing of additional noise on the g vector. S vector= ((sigma(noise))
c

         WRITE(*,*) '-----------------------------------------------'
         WRITE(*,*) ' *********   gnoise = g + noise(sigma) '
         WRITE(*,*) '-----------------------------------------------'
         WRITE(*,*)


         factor=1.00
c         call gaussn2(g,gnoise,s,sizg,factor)

ccc Computing of the ratio vector of normalized fi and dfi.

         do i=1,sizfi
            ratio(i)=fi(i)/dfi(i)
            sumratio=sumratio+ratio(i)
         end do

cccc        call normalisation(ratio,sizfi,1.)

	end if

c End of conditions on choice1. Common block
 100    continue

c Calculation of convolution matrix F 
c -----------------------------------
c Preliminary direct convolution as a test: g1 can be compared with matF.fi
c        call convol_(f,dfi,g1,q,p,sizg)

c        call calc_matF(f,sizf,matF,p,q,sizg)

	call mcal_mat(f,sizf,matF,p,q,sizg)


c	open(unit=400, file='./plot/testmatf.dat', type='new')
c	i=1
c	do i=1,sizg
c		write(400,*)'Index=',i
c		do k=1,sizfi
c			write(400,*)matF(i,k)
c		end do
c	end do	
c	close(400)

c	open(unit=500, file='./plot/testf.dat', type='new')
c       do i=1,sizg
c                write(400,*)'Index=',i
c                do k=1,sizf
c                        write(500,*)f(k)
c                end do
c       end do
c        close(500)

	write(*,*)'OK, matrix matF is ready....'
c	read(*,*)rep

c	write(*,*)'Trying without default solution.....'
c	goto 110



ccc matrical multiplication: matF' = matF . matdfi
        do i=1,sizg
           summin=0
           do k=1,sizfi
           matF(i,k)=matF(i,k)*dfi(k)
           summin=summin+matF(i,k)
           end do
           gnoise(i)=g(i)

c      write(*,*) 'i,matF()*dfi(),g(),./.,g1(i) :',i,summin,g(i),
c&     summin/g(i),g1(i)
      	end do

	
c	open(unit=678, file='./plot/testmatf.dat', type='new')
c	chi2=0.
c	do i=1,sizg
c		gnoise(i)=0.
c		do k=1,sizfi
c			gnoise(i)=gnoise(i)+matF(i,k)
c		end do
c		gnoise(i)=gnoise(i)/q
c		write(678,*)g(i),gnoise(i)
c		chi2=chi2+((g(i)-gnoise(i))/s(i))**2.	
c	end do
c	close(678)

c	write(*,*)'Matrix is now transformed back into the g_simu vector!'
c	write(*,*)'Chi2 is now:',chi2
c	read(*,*)rep






110	continue

c==================================================================     
c     Solve vector fi in the system g = F. fi, using the ME method
c     ------------------------------------------------------------
c==================================================================

      WRITE(*,*) 
      WRITE(*,*) '========================================'
      WRITE(*,*) 'Solve vector fi in the system g = F. fi '
      WRITE(*,*) '========================================'
      aff=10
      summin=1000.
      etamin=0.
      niterm=0.
      tstM=1.00

 1    continue

c If using simulated g data, on can choose betweeen a direct resolution
c or a loopinf 

      if (choice1.eq.1) then
        write(*,*) 
        write(*,*) ' 1- Loop with etas'
        write(*,*) ' 2- Resolution of the system'
        read(5,*) choice2
      else 
	choice2=2
      end if

      if (choice2.eq.1) then
c 
      write(*,*) 'Borne min de eta:'
      read(5,*) eta
      write(*,*) 'Borne max de eta:'
      read(5,*) etamax
      write(*,*) 'step:'
      read(5,*) stepet
      write(*,*) ' Nombre d''iterations:'
      read(5,*) niter
c
 2    continue

      write(*,*)
      write(*,*) '************************************************'
      write(*,*) ' niter, eta:',niter,eta
c	Martin V.'s QUICK:
c       call quick(X,matF,gnoise,s,sizg,sizfi,niter,eta,1,n1
c&      ,x1,x2,x3,tstM,cutg,aff)

c	Original QUICK:
        call quick(X,matF,g,s,sizg,sizfi,niter,eta,1,n1
&      ,x1,x2,x3)


      eta=eta+stepet
      if (eta.le.etamax) goto 2

      else

      write(*,*) ' Eta, No of Iterations:',Eta,niter
c      read(5,*) Eta, niter

c	Martin V.'s QUICK:
c       call quick(X,matF,gnoise,s,sizg,sizfi,niter,eta,1,n1
c&      ,x1,x2,x3,tstM,cutg,aff)

	write(*,*)'g(13)=',g(13)

c       Original QUICK:
        call quick(X,matF,gnoise,s,sizg,sizfi,niter,eta,1,n1
&      ,x1,x2,x3)


      end if
      
	write(*,*)'Returned from QUICK!'
c	write(*,*)'g(13)=',g(13)

ccc Reconstruction of the fi result vector: fi = matdfi. X
         do i=1,sizfi
            fimem(i)=dfi(i)*X(i)
            fi(i)=X(sizfi+1-i)
         end do

c       call normalisation (fi,sizfi,1.)
c       call normalisation (fimem,sizfi,1.)

	write(*,*)'Normalising results.....'

	phimax=0.
	do i=1,sizfi
			phimax=phimax+fimem(i)
	end do
	do i=1,sizfi
		fimem(i)=fimem(i)/phimax
	end do
	phimax=0.
        do i=1,sizfi
                        phimax=phimax+dfi(i)
        end do
        do i=1,sizfi
                dfi(i)=dfi(i)/phimax
        end do
	
c-------- checking the chi2:
	write(*,*)'Check of chi2....'
	
c	goto 77

c-------------

	




	write(*,*)'Output to data-files (y/n)?'
	read(*,*)rep

	if (rep.eq.'y') then

	open(unit=554, file='./plot/dfi.dat', type='new')
        do i=1,sizfi
                write(554,*)dfi(i)
        end do
        close(554)
        write(*,*)'Vector dfi written to file dfi.dat'

	open(unit=555, file='./plot/x.dat', type='new')
        do i=1,sizfi
                write(555,*)X(i)
        end do
        close(555)
        write(*,*)'Vector X written to file x.dat'

	open(unit=556, file='./plot/phi.dat', type='new')
	do i=1,sizfi
		write(556,*)fimem(i)
	end do
	close(556)
	write(*,*)'Vector Phi*X written to file phi.dat'

	end if

c      write (6,*) '* i      FI(i)      FIdef(i)  FImem  *'  
c      do k=1,sizfi
c         write (6,*) k,FI(k),dfi(k),fimem(k)
c      end do

      write(*,*)
      write(*,*) 'Another loop params_opt & MEM, using fi as IP (y/n)?'
      read(5,*) rep
      if (rep.eq.'y') then

		write(*,*)'Value of Default Phi dfi(13)=',dfi(13)
		write(*,*)'Value of MEM Phi fimem(13)=',fimem(13)
         do i=1,sizfi
            dfi(i)=fimem(i)
         end do
	loop=loop+1
	write(*,*)'NEW Value dfi(13)=',dfi(13)
	write(*,*)'Loop is now:',loop
	write(*,*)'Returning to params_opt.......'

c        call params_opt(g,sizg,wlg,s,dfi,sizfi,f,par,sizf,loop)

        goto 100
      end if

c      write(*,*) ' Name of the resulting file ?'
c      read(5,*) nmfresfi
c      open(300,file='./plot/'//nmfresfi)
c      open(400,file='./data/resfi')
c      write(400,*) sizfi,0
c       do k=1,sizfi
c          write(300,*) FImem(k),dfi0(k)
c          write(400,*) FImem(k)
c       end do
c      close(300)
c      close(400)

	write(*,*)'Quit IP program (y/n)?'
	read(5,*) rep
	if (rep.eq.'y') then
		write(*,*)'Adios! Amigo......'
		return	
	end if

	if (rep.eq.'n') then
		goto 1
	end if

      goto 1
      stop

 200  continue



c*****************************************************************************
c Solve vector f in hte system g = FI . f, using the ME method
c*****************************************************************************

c     Values of matrix FI 
c     -------------------
       sizfi=2*p+1
c
ccc Test with simulated values of vector fi
c        write(*,*) ' ** ok5'
        open (400,file='../data/resfi')
        read (400,*) sizfi,wlf
c        write (*,*) sizfi,wlf
        do k=1,sizfi
           read (400,*) fi(k)
        end do
        write(*,*) ' ** ok5'
        close (400)
c
     
ccc Linear interpolation of g vector to set the default f vector

        sumf=0.
        g(sizg+1)=g(sizg)

        do i=1,sizg
           do j=1,5
              k=(i-1)*5+j
              deff(k)=(g(i)+(g(i+1)-g(i))/5*(j-1))/5
              write(*,*) ' k, g(k)/5., inter(f)(k) :',k,g(i)/5.,deff(k)
              sumf=sumf+deff(k)
           end do
        end do
ccc
c Size of f vector
         sizf=sizg*q          

         cutg=4
c         call params_opt(deff,f,deff,sizf,signoise,cutg,shift)
        
       write(*,*) ' ** ok5'

       do i=1,sizg
c
        do k=q*(i-1)+1-p,q*i+p
       
           m=q*(i-1)-k+p
           n=q*i-k+p-1
           sum=0.
           do l=m,n
c condition so that FI(i-j) is taken in the interval [-p..p]
c (i.e. [1..2p+1] with the vectorial indice l for FI])
              if (l.ge.1.and.j.le.sizfi) then
              sum=sum+fi(l)
              end if
           end do 
           if (k.gt.0.and.k.le.sizf) then
           matFI(i,k)=sum*deff(k)

c           write (6,*) ' FI (',i,',',k,') =',matFI(i,k)
c           else 
c              matFI(i,k)=0.
           end if
        end do
        end do

        tstM=1.
        cutg=4
        do j=1,10

      write(*,*) ' Eta, No of Iterations ?'
      read(5,*) Eta, niter
c
c       call quick(X,matFI,gnoise,s,sizg,sizf,niter,eta,1,n1
c&      ,x1,x2,x3,tstM,cutg,1)

       sum=0.
       do i=1,sizf
          f2(i)=X(i)*deff(i)*q
          sum=sum+f2(i)
       end do
c
       write(*,*) ' sum(f2), sum(f), sum(f)/sum(f2)=',sum,sumf,sumf/sum
       
       do i=1,sizf
          f2(i)=f2(i)*sumf/sum
          write(*,*) 'i,f2(i),f(i),f2(i)/f(i)',i,f2(i),f(i),f2(i)/f(i)          
       end do

       open(300,file='../midas/resf.dat')
       do k=1,sizf
          write(300,*) f2(k),f(k),deff(k)
       end do
       close(300)

       end do
c=============== END OF RV RUN ==========================
 8000 	CONTINUE

	IF (par(20).eq.1) THEN	
	
	CLOSE(37)


c	old:
cc	chunk=chunk-1
c	new:
	chunk=totalchunks

	WRITE(*,*)'=========================================='
c	write(*,*)
	WRITE(*,*)'     R A D I A L  V E L O C I T Y'
c	write(*,*)
	WRITE(*,*)'=========================================='

	chi2sum=chi2sum/chunk

c	write(*,*)'Mean of CHI^2:',chi2sum
	
c 	write(*,*)
	write(*,*)'Number of Chunks:',chunk

        chunksaf=chunk

        zeler=0

c       DO clip=2.0,2.3,0.1
        DO clip=2.1,3.3,0.1

c        write(*,*)'OK, clip= ',clip
        zeler = zeler + 1


	OPEN(37,file='./plot/chunk/'//outfn//'_rv1.dat',type='old')
	OPEN(39,file='./plot/chunk/'//outfn//'_rv3.dat')	

	radvelsum=0.D0
	radvelmean=0.D0
	gradmax = 0.
	chunk=chunksaf

	do i=1,chunk
c		normal way:
c		read(37,*) order,j,radvel(i)
c		new way:
		read(37,108) order,j,radvel(i),dummy,gradient(i)
		IF (gradient(i).gt.gradmax) gradmax=gradient(i)
c		normal way:
c		radvelsum=radvelsum+radvel(i)
		WRITE(39,*) order,j,barycorr-radvel(i),gradient(i)
	end do

c	open(23,file='./plot/chunk/gradtest2.dat')
	gradsum=0.
	do i=1,chunk
		gradient(i)=gradient(i)/gradmax
		radvelsum=radvelsum+(radvel(i)*gradient(i))
		gradsum=gradsum+gradient(i)
c		write(23,*) i,gradient(i)
	end do
c	CLOSE(23)

	CLOSE(37)
	CLOSE(39)	
 108    FORMAT(I2,1X,I2,1X,E25.15,1X,E25.15,1X,E25.15)

c	normal way:
c	radvelmean=radvelsum/DBLE(chunk)
	
c	weighted mean:
	radvelmean=radvelsum/gradsum

	write(*,*)'=========> Mean velocity:',radvelmean,' m.s-1'

	radvelvar=0.D0

	do i=1,chunk
		radvelvar=radvelvar+(radvel(i)-radvelmean)**2.d0 
	end do

	radvelvar=radvelvar/DBLE(chunk-1)

	radvelrms=SQRT(radvelvar)

	write(*,*)'=========> RMS: ',radvelrms,' m.s-1'

	radvelmeanerr=radvelrms/SQRT(DBLE(chunk))

	write(*,*)'=========> ERROR OF MEAN RV: ',radvelmeanerr,' m.s-1'

	write(*,*)'Now eliminating outliers.....'

 8700	CONTINUE
	count=0
	outie=0

	do i=1,chunk
		radresid=ABS(radvelmean-radvel(i))

c================================================================================
c	TRASHING OF OUTLIERS:
c================================================================================

c	Glamdring:
c		if (radresid.lt.radvelrms*2.3D0) then

c	2008-07-22: 
c	OPEN(98,file='clip',type='old')
c		READ(98,*) clip
c	CLOSE(98)

	if (radresid.lt.radvelrms*clip) then

c	99.9 %:
c		if (radresid.lt.radvelrms*3.2905) then
c		if (radresid.lt.radvelrms*2.7D0) then
c	99 %:
c		if (radresid.lt.radvelrms*2.58D0) then
c	95 %:
c		if (radresid.lt.radvelrms*1.96D0) then

			count=count+1
			radvel2(count)=radvel(i)
			gradient2(count)=gradient(i)
			else
			outie=outie+1
			write(*,*)outie,' Outies trashed....'
		end if

	end do

	radvelsum=0.D0
	gradsum=0.D0

	do i=1,count
		radvel(i)=radvel2(i)
		gradient(i)=gradient2(i)
		radvelsum=radvelsum+radvel(i)*gradient(i)	 
		gradsum=gradsum+gradient(i)
	end do

c	weighted mean:
	radvelmean=radvelsum/gradsum

c	normal mean:
c	radvelmean=radvelsum/DBLE(count)

	radvelvar=0.D0	
	do i=1,count
                radvelvar=radvelvar+(radvel(i)-radvelmean)**2.d0
        end do
	radvelvar=radvelvar/DBLE(count-1)
	radvelrms=SQRT(radvelvar)

	if (outie.gt.0) then
		chunk=count
		goto 8700
	end if

	radvelmeanerr=radvelrms/SQRT(DBLE(count))

        DO i=1,count
        XR(i)=radvel(i)
c       WRITE(*,*)'i= ',i,' radvel= ',radvel(i),' XR= ',XR(i)
        END DO
        NR=count
c       WRITE(*,*)'NR= ',NR
        XMED=0.
 
        CALL SORT(count,XR)
 
c        DO i=1,count
c        WRITE(*,*)'i= ',i,' radvel= ',radvel(i)
c        END DO
 
        CALL MDIAN1(XR,NR,XMED)
 
        WRITE(*,*)'XMED= ',XMED
        radmedian=XMED

        rvsaf(zeler) = radvelmean
        rvmsaf(zeler) = radmedian
        rmssaf(zeler) = radvelrms
        errsaf(zeler) = radvelmeanerr

c------- clip loop 2.0 -> 3.3

        END DO

c        write(*,*)'zeler = ',zeler

        rvfinal=0.d0
        rvmfinal=0.d0
        errfinal=0.d0
        rmsfinal=0.d0

        DO i=1,zeler
c               WRITE(*,*) rvsaf(i),rvmsaf(i),errsaf(i),rmssaf(i)
                rvfinal=rvfinal+rvsaf(i)
                rvmfinal=rvmfinal+rvmsaf(i)
                errfinal=errfinal+errsaf(i)
                rmsfinal=rmsfinal+rmssaf(i)
        END DO

        rvfinal=rvfinal/DBLE(zeler)
        rvmfinal=rvmfinal/DBLE(zeler)
        errfinal=errfinal/DBLE(zeler)
        rmsfinal=rmsfinal/DBLE(zeler)

c       WRITE(*,*)'OK, final:'
c       WRITE(*,*) rvfinal,rvmfinal,errfinal,rmsfinal

        radvelmean=rvfinal
        radmedian=rvmfinal
        radvelrms=rmsfinal
        radvelmeanerr=errfinal



	write(*,*)'Chunks used:',count 
	write(*,*)'Sigma clip: ',clip
	write(*,*)'Okay, new values:'
	write(*,*)'---> RV (weighted mean): ',radvelmean
        write(*,*)'---> RV (median): ',radmedian
c	write(*,*)'---> final average RV: ',(radvelmean+radmedian)/2.D0        
	write(*,*)'---> RMS: ',radvelrms,' m.s-1'
	write(*,*)'---> ERROR:',radvelmeanerr,' m.s-1'
	write(*,*)'Barycorr=',barycorr,' m.s-1'
	write(*,*)'True weigthed mean RV: ',barycorr-radvelmean
        write(*,*)'True median RV : ',barycorr-radmedian    
c	write(*,*)'True final (average) RV: ',barycorr-((radvelmean+radmedian)/2.D0)

c	open(23,file='./plot/chunk/gradtest3.dat')
c	do i=1,count
c		write(23,*) i,radvel(i),gradient(i)
c	end do
c	CLOSE(23)


	OPEN(38,file='./plot/chunk/'//outfn//'_rv2.dat')
	do i=1,count
		write(38,101)radvel(i)
	end do
	write(*,*)'cleaned RVs written to outputfile...'
	CLOSE(38)



c	OPEN(38,file='./plot/chunk/result.dat')
c	write(38,*)'Chunks used:',count
c       write(38,*)'Okay, new values:'
c        write(38,*)'---> RV: ',radvelmean
c        write(38,*)'---> RMS: ',radvelrms,' m.s-1'
c        write(38,*)'---> ERROR:',radvelmeanerr,' m.s-1'
c	write(38,*)'Barycorr:',barycorr,' m.s-1'
c	write(38,*)
c	write(38,*)'True RV: ',barycorr-radvelmean 
c
c	CLOSE(38)

c------	WRITING TO OUTPUTFILE (CATALOG)


c---	determining average S/N of analysed orders:

	s2n=0.

	DO i=startorder,endorder
		s2n=s2n+s2n_order(i)
	END DO

c	WRITE(*,*)'Startorder=',startorder
c	WRITE(*,*)'endorder=',endorder

	IF (endorder.ne.startorder) THEN
c		WRITE(*,*)'S/N SUM= ',s2n
c		WRITE(*,*)'(endorder-startorder)+1 = ',(endorder-startorder)+1
		s2n=s2n/((endorder-startorder)+1)
	END IF		

c	WRITE(*,*)'AVERAGE S/N = ',s2n

c---

	OPEN(38,file='./plot/chunk/'//outputfn)

c---	Jumping over existing results
	do i=1,n_analyzed
		read(38,*)dummy
	end do
c---
c	write(*,*)'Now writing to outputfile, okay?'
c	read(*,*)rep

c	new:
c	write(38,3888)BJD,barycorr-((radvelmean+radmedian)/2.D0),barycorr-radmedian,
c&	radvelmeanerr,
c&      radvelrms,radvelmean,s2n,weather,quality,chi2sum,count,spectrumfn

c	now with weigthed mean:
c	original:	
	write(38,3888)BJD,barycorr-radvelmean,barycorr-radmedian,radvelmeanerr,
&	radvelrms,radvelmean,s2n,weather,quality,chi2sum,clip,count,spectrumfn

	n_analyzed=n_analyzed+1

c	write(38,3888)BJD,barycorr-radvelmean

3888	FORMAT(F16.8,1X,F9.2,1X,F9.2,1X,F7.2,1X,F7.2,1X,F9.2,1X,F4.0,1X
&	,I1,1X,I1,1X,F9.2,1X,F4.2,1X,I4,1X,A38)

c3888    FORMAT(E22.15,1X,F8.2)

c3888	FORMAT(E22.15,1X,F8.2,1X,F5.2,1X,F5.2,1X,F4.0,1X,A30)

	CLOSE(38)

c------

	END IF

c===================== THE END OF EVERYTHING ================
 9000   CONTINUE
	
        WRITE(*,*)'EOF OF SPECTRUM REACHED!'
        WRITE(*,*)'....IP-PRG SHUTTING DOWN'
	write(*,*)'**************** T E Q U I L A ! *************'
 
	close(15)
	close(36)
c	close(37)

c	CLOSE(71)

101 	FORMAT(1(3x,e25.15))

      	RETURN
      	END




cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      subroutine normalisation(v,sizv,Dx)
c
c     v      Real 1-D array  Vector to be normalized              In/Out      
c     sizv   Real            Size of vector v
c     Dx     Real            Value of the integration step        Input
c      
c     The integration is made with boxes

c      parameter (maxsiz=1000)
      Implicit none
	integer maxsiz
	parameter (maxsiz=1000) 

    
      Real v(maxsiz)
      Real sum,Dx
      Integer i,sizv

 
      sum=0.
      do i=1,sizv
c          write(*,*) ' v (',i,') =',v(i)      
         sum=sum+v(i)
      end do
      sum=sum*Dx
      do i=1,sizv
        v(i)=v(i)/sum
c        write(*,*) ' vnormal (',i,') =',v(i)
      end do

      return
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      subroutine matvecmul(a,b,c,siza,sizc)
c
c      parameter (maxsiz=1000)
      Implicit none
	Integer maxsiz
	parameter (maxsiz=1000)


      Integer i,j,siza,sizc
      Real sum
c      Real a(siza),c(sizc),b(siza,sizc)
       Real a(maxsiz),c(maxsiz),b(maxsiz,maxsiz)     


c      write (*,*) ' bs, cs :',b(1,3),b(2,2),b(2,3),c(1),c(2),c(3)

      write(*,*) ' ok matvecmul1',siza,sizc
      do i=1,siza

      sum=0.
        do j=1,sizc
c          write(*,*) 'b ,i, c, j:',b(i,j),c(j)
          sum=sum+b(i,j)*c(j)
        end do
        a(i)=sum
c      write(*,*) i,sum,a(i)
      end do
      return
      end

c--------------------------------------------------------------------

      subroutine calc_matF(f,sizf,matF,p,q,sizg)

c=========================================
c     Calculation of convolution matrix F 
c     -----------------------------------
c=========================================

c      parameter (maxsiz=1000)

      implicit none
	integer maxsiz
	parameter (maxsiz=1000)

      real f(maxsiz),matF(maxsiz,maxsiz),sum
      integer i,j,k,m,n,p,q,sizg,sizf

c       sizf=sizg*q

       write(*,*) ' p, q:',p,q

      do i=1,sizg
c         write(*,*) 'i,f(i):',i,f(i)
        do k=1,2*p+1
           m=q*(i-1)+k-p
           n=q*i+k-p-1
           sum=0.
c           write (*,*) 'i,k,m,n :',i,k,m,n
           do j=m,n
c condition for the edge effects
              if (j.gt.0.and.j.le.sizf) then
              sum=sum+f(j)
              end if
           end do
		matF(i,k)=sum
c           matF(i,k)=sum/q
c          write (*,*) ' F (',i,',',k,') =',matF(i,k)
        end do
       end do


       return
       end

c****************************************************************
      subroutine convol_(f,fi,g1,q,p,sizg)

c      parameter (maxsiz=1000)

      implicit none
	integer maxsiz
	parameter (maxsiz=1000)

      real f(maxsiz),fi(maxsiz),g1(maxsiz),sum
      integer i,j,k,p,q,sizg,sizf

      sizf=sizg*q

       do i=1,sizg

           sum=0.
           do k=q*(i-1)+1,q*i       
               do j=k-p,k+p
                if (j.gt.0.and.j.le.sizf) then
c               write(*,*) 'j ,.., fi () :',j,k-j+p+1,fi(k-j+p+1)
                 sum=sum+f(j)*fi(k-j+p+1)
                 end if

               end do
            end do
            g1(i)=sum/q
         end do

         return
         end


c------------------------------------------------------------------------
	SUBROUTINE MCAL_MAT(f,sizf,matF,p,q,sizg)

	IMPLICIT NONE
	INTEGER maxsiz
	PARAMETER(maxsiz=1000)
c	PARAMETER(maxsiz=5000)

	INTEGER sizf,sizg,p,q
c	REAL f(maxsiz)
	REAL f(5000)

	REAL matF(maxsiz,maxsiz)
	INTEGER j,js,i,k
	REAL sumf,jdown,jup

	write(*,*)'This is MCAL_MAT!'
	write(*,*)'Size of f=',sizf
	write(*,*)'Size of g=',sizg
	write(*,*)'Oversampling factor=',q
	write(*,*)'Half size of IP=',p

c	BEWARE: index for IP runs from now from +p to -p 
c	like in the formula

c	Index for the matrix:
	k=1

	do i=1,sizg

c	open(unit=999, file='./plot/testmatf.dat',type='new')
	k=1
		do js=p*2,0,-1
c	js=1
			jdown=((q*i)-(q-1)+p)-(js-p)
			jup=((q*(i+1)-1)-(q-1)+p)-(js-p)
			sumf=0.

c			write(*,*)'jdown=',jdown
c			write(*,*)'jup=',jup
c			write(999,*)'....js=',(js-p)

			do j=jdown,jup
c				write(999,*)'f(',j,')=',f(j)
		
				sumf=sumf+f(j)

			end do
	
			matF(i,k)=sumf
			
c			write(999,*)'matF(',i,'',k,')=',matF(i,k)
			k=k+1
		end do

	end do
	
c	close(999)



	RETURN
	END
 


c----------------------------------------------------------------------------
	SUBROUTINE MDIAN1(XR,NR,XMED)
        DIMENSION XR(NR)
        CALL SORT(NR,XR)
        N2=NR/2
        IF(2*N2.EQ.NR)THEN
                XMED=0.5*(XR(N2)+XR(N2+1))
        ELSE
                XMED=XR(N2+1)
        ENDIF
        RETURN
        END
 
 
        SUBROUTINE SORT(NR,RA)
        DIMENSION RA(NR)
 
c        WRITE(*,*) 'subroutine sorting'
 
c        DO I=1,NR
c        WRITE(*,*) RA(I)
c        END DO
c        WRITE(*,*)'End of sorting..'
 
        LM=NR/2+1
        IR=NR
6789    CONTINUE
        IF(LM.GT.1)THEN
          LM=LM-1
          RRA=RA(LM)
        ELSE
          RRA=RA(IR)
          RA(IR)=RA(1)
          IR=IR-1
          IF(IR.EQ.1)THEN
            RA(1)=RRA
            RETURN
          ENDIF
        ENDIF
        IM=LM
        JM=LM+LM
7766      IF(JM.LE.IR)THEN
          IF(JM.LT.IR)THEN
            IF(RA(JM).LT.RA(JM+1))JM=JM+1
          ENDIF
          IF(RRA.LT.RA(JM))THEN
            RA(IM)=RA(JM)
            IM=JM
            JM=JM+JM
          ELSE
            JM=IR+1
          ENDIF
        GO TO 7766
        ENDIF
        RA(IM)=RRA
        GOTO 6789
        END
