C==============================================================================
      SUBROUTINE QUICK(X,A,Y,S,M,N,NITER,ETA,DIAG,ITN,CHI2,ENTR,TEST)
C==============================================================================
C
C     Description of calling variables :
C     ----------------------------------
C     Name   Type            Description                               transfer
C     -------------------------------------------------------------------------
C     X      Real 1-D array  MEM solution vector                       output
C     A      Real 2-D array  response matrix                           input
C     Y      Real 1-D array  data vector                               input
C     S      Real 1-D array  data error vector                         input
C     M      Integer         number of data points                     input
C     N      Integer         number of solution points                 input
C     NITER  Integer         maximum number (</= 10000) of iterations  input
C     ETA    Real            trade off                                 input
C     DIAG   Integer         on/off (1/0) switch for diagnostics       inout
C     ITN    Integer         number of carried out iterations          output
C     CHI2   Real            chi-square of solution (should be </= M)  output
C     ENTR   Real            entropy of solution                       output
C     TEST   Real            a measure of the non-parallelness of      output
C                            grad(ENTR) and grad(CHI2)
C
C                                   1 |  grad S     grad C  | 2    
C                            TEST = - | -------- - -------- |      
C                                   2 | |grad S|   |grad C| |
C               
C==============================================================================
C
      IMPLICIT NONE
	INTEGER maxsiz
	parameter (maxsiz=1000)

      INTEGER I,K,L,N,M,ITN,NITER,DIAG
      REAL SUM1,SUM2,SUM4,QNORM,ETA,CHI2,CHIOLD,ENTR,ENOLD
      REAL X(maxsiz),S(maxsiz),Y(maxsiz),A(maxsiz,maxsiz),X2(maxsiz)
      REAL XOLD(maxsiz)
C-----
      REAL SUMY,SUMY0,QNORMY,Y0(maxsiz)
      REAL SUM4A,GRADC(maxsiz),GRADE(maxsiz)
      REAL QA,QNE,QNC,TEST,TOLD,SUMYY,SUMYC

C	write(*,*)'MAXSIZ=',maxsiz
C
C     Algorithm works with normalized images   SUM X(K) = 1.0 ,
C     therefore data and errors will be normalized accordingly :
C     ----------------------------------------------------------
C
      SUMY0 = 0.0
      SUMY = 0.0
      DO I=1,M,1
         Y0(I) = 0.0
         DO K=1,N,1
*            Y0(I) = Y0(I)+A(I,K)/FLOAT(N)
            Y0(I) = Y0(I)+A(I,K)
         ENDDO
         SUMY0 = SUMY0+Y0(I)
         SUMY  = SUMY+Y(I)
      ENDDO
*      QNORMY = SUMY0/SUMY
      QNORMY = SUMY0/SUMY/FLOAT(N)
CCC      WRITE(*,*) '=== QNORMY = ',QNORMY
      DO I=1,M,1
         Y(I) = Y(I)*QNORMY
         S(I) = S(I)*QNORMY
      ENDDO
C
CCC      IF (DIAG.EQ.1) THEN
         WRITE(*,*) '------------------------------------------------'
         WRITE(*,*) ' Iter.  chi-square   # data pts   Entropy   Test'
         WRITE(*,*) '------------------------------------------------'
CCC      ELSE
CCC         WRITE(*,*) '------------------------------------------------'
CCC         WRITE(*,*) ' Iter.  chi-square   # data pts                 '
CCC         WRITE(*,*) '------------------------------------------------'
CCC      ENDIF
C
C     Algorithm :
C     -----------
C
C     1st step :
C     ----------
      ITN = 1
C
      DO K=1,N,1
         X2(K)= 1.0/FLOAT(N)
*         X(K) = X2(K)/QNORMY
      ENDDO
C
C     Chi-square :
C     ------------
      CHI2 = 0.0
      DO I=1,M,1
         SUM4 = 0.0
         DO K=1,N,1
            SUM4 = SUM4+A(I,K)*X2(K)
         ENDDO
         CHI2 = CHI2+((SUM4-Y(I))/S(I))**2
      ENDDO
C
      IF (DIAG.EQ.1) THEN
C
C        Partial derivatives of chi-square :
C        -----------------------------------
         DO L=1,N,1
            QA = 0.0
            DO I=1,M,1
               SUM4A = 0.0
               DO K=1,N,1
                  SUM4A = SUM4A+A(I,K)*X2(K)
               ENDDO
               QA = QA+((SUM4A-Y(I))*A(I,L)/S(I))**2
            ENDDO
            GRADC(L) = 2.0*QA
         ENDDO
C
C        Entropy :
C        ---------
         ENTR = 0.0
         DO K=1,N,1
            ENTR = ENTR-X2(K)*LOG(X2(K))
         ENDDO
C
C        Partial derivatives of entropy :
C        --------------------------------
         DO K=1,N,1
            GRADE(K) = -1.0-LOG(X2(K))
         ENDDO
C              
C        Test :
C        ------
         QNE = 0.0
         QNC = 0.0
         DO K=1,N,1
            QNE = QNE+GRADE(K)**2
            QNC = QNC+GRADC(K)**2
         ENDDO
         QNE = SQRT(QNE)
         QNC = SQRT(QNC)
         TEST = 0.0
         IF ((QNC.GT.0.0).AND.(QNE.GT.0.0)) THEN
            DO K=1,N,1
               TEST = TEST+(GRADE(K)/QNE - GRADC(K)/QNC)**2
            ENDDO
            TEST = 0.5*TEST
         ELSE
            TEST = 99999999.
         ENDIF
C
ccc michl: comment output:
         WRITE(*,*) ITN,CHI2,M,ENTR,TEST
C
         ENOLD  = ENTR
         TOLD   = TEST
C
      ELSE
         WRITE(*,*) ITN,CHI2,M
      ENDIF
C
      CHIOLD = CHI2
C
c***** next line not valid since chi2 < M
c	IF (CHI2.LE.M) GOTO 2223
c*****************************************
C
C-------------------
      ETA = -2.0*ETA
C-------------------
C
C        Loop for on-line algorithm :
C        ----------------------------
         DO ITN = 2,NITER,1
            DO K=1,N,1
               SUM2 = 0.0 
               DO I=1,M,1
                  SUM1 = 0.0
                  DO L=1,N,1
                     SUM1 = SUM1+A(I,L)*X2(L)
                  ENDDO
                  SUM2 = SUM2+(SUM1-Y(I))*A(I,K)/S(I)/S(I)
               ENDDO 
               XOLD(K) = X2(K)
*              X2(K) = XOLD(K)*EXP(ETA*SUM2)
               X2(K) = XOLD(K)*(1.0+ETA*SUM2)  
            ENDDO
            QNORM = 0.0
            DO K=1,N,1
               QNORM = QNORM+X2(K)
            ENDDO
            DO K=1,N,1
               X2(K) = X2(K)/QNORM
*               X(K) = X2(K)/QNORMY
            ENDDO
C
C           Update norm of data vector :
C           ----------------------------
            SUMYC = 0.0
            SUMYY = 0.0
            DO I=1,M,1
               DO K=1,N,1
                  SUMYC = SUMYC+A(I,K)*X2(K)
               ENDDO
               SUMYY = SUMYY+Y(I)
            ENDDO 
C
c            WRITE(*,*) '--- Current    sum of y-s = ',SUMYY
c            WRITE(*,*) '--- Calculated sum of y-s = ',SUMYC
C
            DO I=1,M,1
               Y(I) = Y(I)*SUMYC/SUMYY
               S(I) = S(I)*SUMYC/SUMYY
            ENDDO
C
C           Chi-square :
C           ------------
            CHI2 = 0.0
            DO I=1,M,1
               SUM4 = 0.0
               DO K=1,N,1
                  SUM4 = SUM4+A(I,K)*X2(K)
               ENDDO
               CHI2 = CHI2+((SUM4-Y(I))/S(I))**2
            ENDDO
C
            IF (DIAG.EQ.1) THEN
C
C              Partial derivatives of chi-square :
C              -----------------------------------
               DO L=1,N,1
                  QA = 0.0
                  DO I=1,M,1
                     SUM4A = 0.0
                     DO K=1,N,1
                        SUM4A = SUM4A+A(I,K)*X2(K)
                     ENDDO
                     QA = QA+((SUM4A-Y(I))*A(I,L)/S(I))**2
                  ENDDO
                  GRADC(L) = 2.0*QA
               ENDDO
C
C              Entropy :
C              ---------
               ENTR = 0.0
               DO K=1,N,1
                  ENTR = ENTR-X2(K)*LOG(X2(K))
               ENDDO
C
C              Partial derivatives of entropy :
C              --------------------------------
               DO K=1,N,1
                  GRADE(K) = -1.0-LOG(X2(K))
               ENDDO
C
C              Test :
C              ------
               QNE = 0.0
               QNC = 0.0
               DO K=1,N,1
                  QNE = QNE+GRADE(K)**2
                  QNC = QNC+GRADC(K)**2
               ENDDO
               QNE = SQRT(QNE)
               QNC = SQRT(QNC)
               TEST = 0.0
               IF ((QNC.GT.0.0).AND.(QNE.GT.0.0)) THEN
                  DO K=1,N,1
                     TEST = TEST+(GRADE(K)/QNE - GRADC(K)/QNC)**2
                  ENDDO
                  TEST = 0.5*TEST
               ELSE
                  TEST = 99999999.
               ENDIF
C
ccc	michl comments this out:
               WRITE(*,*) ITN,CHI2,M,ENTR,TEST
C
               ENOLD  = ENTR
               TOLD   = TEST
            ELSE
               WRITE(*,*) ITN,CHI2,M
            ENDIF
C
            CHIOLD = CHI2
C
c**************************************
c           IF (CHI2.LE.M) GOTO 2223 
c**************************************
C
         ENDDO 
C
 2222 CONTINUE

c	open(unit=55, file='./plot/x.dat', type='new')
c        do i=1,49
c                write(55,*)X2(i)
c        end do 
c        close(55)
c        write(*,*)'Vector X written to file x.dat'
c	read(*,*)ITN

C
C
C     Return un-normalized output image:
C     ----------------------------------
C
      ITN  = ITN-1
      CHI2 = CHIOLD
      IF (DIAG.EQ.1) THEN
         ENTR = ENOLD
         TEST = TOLD
      ELSE
         ENTR = 0.0
         TEST = 0.0
      ENDIF
*      DO K=1,N,1
*         X(K) = XOLD(K)/QNORMY
*      ENDDO
C
 2223 CONTINUE
C
*      SUMYY = 0.0
*      DO I=1,M,1
*         SUMYY = SUMYY+Y(I)
*      ENDDO
C
      DO K=1,N,1
         X(K) = X2(K)*SUMY/SUMYC
      ENDDO

	DO I=1,M,1
	  Y(i)=Y(i)*SUMY/SUMYC
	  S(i)=S(i)*SUMY/SUMYC
	END DO
	
	ETA=-0.5*ETA

C
 1111 FORMAT(6(1X,F12.7))
C
      RETURN
      END
